@@ -14,9 +14,8 @@ from lpipe.exceptions import FailButContinue
 def test_func(foo: str, logger, event, **kwargs):
     if not foo:
         raise Exception("Missing required parameter 'foo'")
-    assert isinstance(event, dict)
-    assert event["context"].function_name == "my_lambda"
-    assert isinstance(event["payload"], Payload)
+    assert event.context.function_name == "my_lambda"
+    assert isinstance(event.payload, Payload)
     logger.log("test_func success")
     return True
 
@@ -32,18 +31,26 @@ def test_func_default_param(logger, foo: str = "bar", **kwargs):
 
 
 def test_func_trigger_first(logger, **kwargs):
-    return Payload(Path.TEST_TRIGGER_SECOND, kwargs)
+    return Payload(path=Path.TEST_TRIGGER_SECOND, kwargs={})
 
 
 def test_func_multi_trigger(logger, **kwargs):
     return [
-        Payload(Path.TEST_TRIGGER_SECOND, kwargs),
-        Payload(Path.TEST_TRIGGER_SECOND, kwargs),
+        Payload(path=Path.TEST_TRIGGER_SECOND, kwargs={}),
+        Payload(path=Path.TEST_TRIGGER_SECOND, kwargs={}),
+        Payload(
+            queue=Queue(
+                name=config("TEST_KINESIS_STREAM"),
+                type=QueueType.KINESIS,
+                path="TEST_FUNC",
+            ),
+            kwargs={"foo": "bar"},
+        ),
     ]
 
 
 def test_func_trigger_error(logger, **kwargs):
-    return Payload(Path.TEST_RAISE, kwargs)
+    return Payload(path=Path.TEST_RAISE, kwargs={})
 
 
 def return_foobar(**kwargs):
@@ -86,18 +93,19 @@ class Path(Enum):
     MULTI_TEST_FUNC = 8
     MULTI_TEST_FUNC_NO_PARAMS = 9
     TEST_RENAME_PARAM = 10
-    TEST_KINESIS_PATH = 11
-    TEST_SQS_PATH = 12
-    TEST_SENTRY = 13
-    TEST_RET = 14
-    TEST_TRIGGER_FIRST = 15
-    TEST_TRIGGER_SECOND = 16
-    TEST_MULTI_TRIGGER = 17
-    TEST_TRIGGER_ERROR = 18
-    TEST_DEFAULT_PATH = 19
-    TEST_DEFAULT_PATH_INCLUDE_ALL = 20
-    TEST_BARE_FUNCS = 21
-    TEST_RAISE = 22
+    TEST_KINESIS_QUEUE = 11
+    TEST_SQS_QUEUE = 12
+    TEST_SQS_QUEUE_WITHOUT_PATH = 13
+    TEST_SENTRY = 14
+    TEST_RET = 15
+    TEST_TRIGGER_FIRST = 16
+    TEST_TRIGGER_SECOND = 17
+    TEST_MULTI_TRIGGER = 18
+    TEST_TRIGGER_ERROR = 19
+    TEST_DEFAULT_PATH = 20
+    TEST_DEFAULT_PATH_INCLUDE_ALL = 21
+    TEST_BARE_FUNCS = 22
+    TEST_RAISE = 23
 
 
 PATHS = {
@@ -130,10 +138,10 @@ PATHS = {
             functions=[test_func],
         )
     ],
-    Path.TEST_KINESIS_PATH: [
+    Path.TEST_KINESIS_QUEUE: [
         Action(
             required_params=["uri"],
-            paths=[
+            queues=[
                 Queue(
                     name=config("TEST_KINESIS_STREAM"),
                     type=QueueType.KINESIS,
@@ -142,16 +150,22 @@ PATHS = {
             ],
         )
     ],
-    Path.TEST_SQS_PATH: [
+    Path.TEST_SQS_QUEUE: [
         Action(
             required_params=["uri"],
-            paths=[
+            queues=[
                 Queue(
                     name=config("TEST_SQS_QUEUE"), type=QueueType.SQS, path="TEST_FUNC"
                 )
             ],
         )
     ],
+    Path.TEST_SQS_QUEUE_WITHOUT_PATH: [
+        Action(
+            required_params=["uri"],
+            queues=[Queue(name=config("TEST_SQS_QUEUE"), type=QueueType.SQS)],
+        )
+    ],
     Path.TEST_FUNC_DEFAULT_PARAM: [Action(functions=[test_func_default_param])],
     Path.TEST_SENTRY: [Action(functions=[throw_exception])],
     Path.TEST_RET: [Action(functions=[return_foobar])],
