@@ -4,7 +4,7 @@ import warnings
 from collections import defaultdict, namedtuple
 from enum import Enum, EnumMeta
 from types import FunctionType
-from typing import Any
+from typing import Any, Union
 
 import lpipe.exceptions
 import lpipe.logging
@@ -14,6 +14,8 @@ from lpipe.contrib import kinesis, mindictive, sqs
 from lpipe.payload import Payload
 from lpipe.queue import Queue, QueueType
 
+PayloadEvent = namedtuple("Event", ["event", "context", "payload"])
+
 
 def build_event_response(n_records, n_ok, logger) -> dict:
     response = {
@@ -28,10 +30,11 @@ def build_event_response(n_records, n_ok, logger) -> dict:
 def process_event(
     event,
     context,
-    paths: dict,
     queue_type: QueueType,
+    paths: dict = None,
     path_enum: EnumMeta = None,
-    default_path=None,
+    default_path: Union[str, Enum] = None,
+    call: FunctionType = None,
     logger=None,
     debug: bool = False,
     exception_handler: FunctionType = None,
@@ -41,10 +44,11 @@ def process_event(
     Args:
         event: https://docs.aws.amazon.com/lambda/latest/dg/python-handler.html
         context: https://docs.aws.amazon.com/lambda/latest/dg/python-handler.html
-        paths (dict): Keys are path names / enums and values are a list of Action objects
         queue_type (QueueType): The event source type.
+        paths (dict): Keys are path names / enums and values are a list of Action objects
         path_enum (EnumMeta): An Enum class which define the possible paths available in this lambda.
-        default_path: A string or Enum which will be run for every message received.
+        default_path (Union[str, Enum]): The path to be run for every message received.
+        call (FunctionType): A callable which, if set and `paths` is not, will disable directed-graph workflow features and default to calling this
         logger:
         debug (bool):
         exception_handler (FunctionType): A function which will be used to capture exceptions (e.g. contrib.sentry.capture)
@@ -59,6 +63,15 @@ def process_event(
             f"Invalid queue type '{queue_type}'"
         ) from e
 
+    if isinstance(call, FunctionType):
+        if not paths:
+            default_path = "AUTO_PATH"
+            paths = {default_path: [call]}
+        else:
+            raise lpipe.exceptions.InvalidConfigurationError(
+                "If you initialize lpipe with a function/callable, you cannot define paths, as you have disabled the directed-graph interface."
+            )
+
     paths, path_enum = normalize.normalize_path_enum(path_enum=path_enum, paths=paths)
 
     successful_records = []
@@ -83,7 +96,9 @@ def process_event(
                 _path = default_path if default_path else _payload["path"]
                 _kwargs = _payload if default_path else _payload["kwargs"]
                 _event_source = get_event_source(queue_type, encoded_record)
-                payload = Payload(_path, _kwargs, _event_source).validate(path_enum)
+                payload = Payload(
+                    path=_path, kwargs=_kwargs, event_source=_event_source
+                ).validate(path_enum)
             except AssertionError as e:
                 raise lpipe.exceptions.InvalidPayloadError(
                     "'path' or 'kwargs' missing from payload."
@@ -155,7 +170,7 @@ def execute_payload(
     debug: bool = False,
     exception_handler: FunctionType = None,
 ) -> Any:
-    """Execute functions, paths, and shortcuts in a Path.
+    """Given a Payload, execute Actions in a Path and fire off messages to the payload's Queues.
 
     Args:
         payload (Payload):
@@ -172,88 +187,42 @@ def execute_payload(
 
     ret = None
 
-    if not isinstance(payload.path, path_enum):
+    if payload.path is not None and not isinstance(payload.path, path_enum):
         payload.path = normalize.normalize_path(path_enum, payload.path)
 
     if isinstance(payload.path, Enum):  # PATH
         paths[payload.path] = normalize.normalize_actions(paths[payload.path])
 
         for action in paths[payload.path]:
-            assert isinstance(action, Action)
+            ret = execute_action(
+                payload=payload,
+                path_enum=path_enum,
+                paths=paths,
+                action=action,
+                logger=logger,
+                event=event,
+                context=context,
+                debug=debug,
+                exception_handler=exception_handler,
+            )
 
-            # Build action kwargs and validate type hints
-            try:
-                dummy = ["logger", "event"]
-                action_kwargs = build_action_kwargs(
-                    action, {**{k: None for k in dummy}, **payload.kwargs}
-                )
-                for k in dummy:
-                    action_kwargs.pop(k, None)
-            except (TypeError, AssertionError) as e:
-                raise lpipe.exceptions.InvalidPayloadError(
-                    f"Failed to run {payload.path.name} {action} due to {utils.exception_to_str(e)}"
-                ) from e
-
-            # Run action functions
-            for f in action.functions:
-                assert isinstance(f, FunctionType)
-                try:
-                    # TODO: if ret, set _last_output
-                    _log_context = {"path": payload.path.name, "function": f.__name__}
-                    with logger.context(bind={**_log_context, "kwargs": action_kwargs}):
-                        logger.log("Executing function.")
-                    with logger.context(bind=_log_context):
-                        ret = f(
-                            **{
-                                **action_kwargs,
-                                "logger": logger,
-                                "event": {
-                                    "event": event,
-                                    "context": context,
-                                    "payload": payload,
-                                },
-                            }
-                        )
-                    ret = return_handler(
-                        ret, path_enum, paths, logger, event, context, debug
-                    )
-                except lpipe.exceptions.LPBaseException:
-                    # CAPTURES:
-                    #    lpipe.exceptions.FailButContinue
-                    #    lpipe.exceptions.FailCatastrophically
-                    raise
-                except Exception as e:
-                    logger.error(
-                        f"Skipped {payload.path.name} {f.__name__} due to unhandled Exception. This is very serious; please update your function to handle this. Reason: {utils.exception_to_str(e)}"
-                    )
-                    if exception_handler:
-                        exception_handler(e)
-                    if debug:
-                        raise lpipe.exceptions.FailCatastrophically() from e
-
-            # Run action paths / shortcuts
-            for path_descriptor in action.paths:
-                _payload = Payload(
-                    normalize.normalize_path(path_enum, path_descriptor),
-                    action_kwargs,
-                    payload.event_source,
-                ).validate(path_enum)
-                ret = execute_payload(
-                    _payload, path_enum, paths, logger, event, context, debug
-                )
-    elif isinstance(payload.path, Queue):  # SHORTCUT
-        queue = payload.path
+    elif isinstance(payload.queue, Queue):  # QUEUE (aka SHORTCUT)
+        queue = payload.queue
         assert isinstance(queue.type, QueueType)
+        if queue.path:
+            record = {"path": queue.path, "kwargs": payload.kwargs}
+        else:
+            record = payload.kwargs
         with logger.context(
             bind={
                 "path": queue.path,
                 "queue_type": queue.type,
                 "queue_name": queue.name,
-                "kwargs": payload.kwargs,
+                "record": record,
             }
         ):
             logger.log("Pushing record.")
-        put_record(queue=queue, record={"path": queue.path, "kwargs": payload.kwargs})
+        put_record(queue=queue, record=record)
     else:
         logger.info(
             f"Path should be a string (path name), Path (path Enum), or Queue: {payload.path})"
@@ -262,8 +231,111 @@ def execute_payload(
     return ret
 
 
+def execute_action(
+    payload: Payload,
+    path_enum: EnumMeta,
+    paths: dict,
+    action: Action,
+    logger,
+    event,
+    context,
+    debug: bool = False,
+    exception_handler: FunctionType = None,
+):
+    """Execute functions, paths, and queues (shortcuts) in an Action.
+
+    Args:
+        payload (Payload):
+        path_enum (EnumMeta): An Enum class which define the possible paths available in this lambda.
+        paths (dict): Keys are path names / enums and values are a list of Action objects
+        action: (Action):
+        logger:
+        event: https://docs.aws.amazon.com/lambda/latest/dg/python-handler.html
+        context: https://docs.aws.amazon.com/lambda/latest/dg/python-handler.html
+        debug (bool):
+        exception_handler (FunctionType): A function which will be used to capture exceptions (e.g. contrib.sentry.capture)
+    """
+    assert isinstance(action, Action)
+    ret = None
+
+    # Build action kwargs and validate type hints
+    try:
+        dummy = ["logger", "event"]
+        action_kwargs = build_action_kwargs(
+            action, {**{k: None for k in dummy}, **payload.kwargs}
+        )
+        for k in dummy:
+            action_kwargs.pop(k, None)
+    except (TypeError, AssertionError) as e:
+        raise lpipe.exceptions.InvalidPayloadError(
+            f"Failed to run {payload.path.name} {action} due to {utils.exception_to_str(e)}"
+        ) from e
+
+    default_kwargs = {
+        "logger": logger,
+        "event": PayloadEvent(event=event, context=context, payload=payload),
+    }
+
+    # Run action functions
+    for f in action.functions:
+        assert isinstance(f, FunctionType)
+        try:
+            # TODO: if ret, set _last_output
+            _log_context = {"path": payload.path.name, "function": f.__name__}
+            with logger.context(bind={**_log_context, "kwargs": action_kwargs}):
+                logger.log("Executing function.")
+            with logger.context(bind=_log_context):
+                ret = f(**{**action_kwargs, **default_kwargs})
+            ret = return_handler(
+                ret=ret,
+                path_enum=path_enum,
+                paths=paths,
+                logger=logger,
+                event=event,
+                context=context,
+                debug=debug,
+            )
+        except lpipe.exceptions.LPBaseException:
+            # CAPTURES:
+            #    lpipe.exceptions.FailButContinue
+            #    lpipe.exceptions.FailCatastrophically
+            raise
+        except Exception as e:
+            logger.error(
+                f"Skipped {payload.path.name} {f.__name__} due to unhandled Exception. This is very serious; please update your function to handle this. Reason: {utils.exception_to_str(e)}"
+            )
+            if exception_handler:
+                exception_handler(e)
+            if debug:
+                raise lpipe.exceptions.FailCatastrophically(
+                    utils.exception_to_str(e)
+                ) from e
+
+    payloads = []
+    for _path in action.paths:
+        payloads.append(
+            Payload(
+                path=normalize.normalize_path(path_enum, _path),
+                kwargs=action_kwargs,
+                event_source=payload.event_source,
+            ).validate(path_enum)
+        )
+
+    for _queue in action.queues:
+        payloads.append(
+            Payload(
+                queue=_queue, kwargs=action_kwargs, event_source=payload.event_source
+            ).validate()
+        )
+
+    for p in payloads:
+        ret = execute_payload(p, path_enum, paths, logger, event, context, debug)
+
+    return ret
+
+
 def return_handler(
-    ret, path_enum: EnumMeta, paths: dict, logger, event, context, debug: bool
+    ret: Any, path_enum: EnumMeta, paths: dict, logger, event, context, debug: bool
 ) -> Any:
     if not ret:
         return ret
@@ -278,17 +350,19 @@ def return_handler(
     except Exception as e:
         logger.debug(utils.exception_to_str(e))
         raise lpipe.exceptions.FailButContinue(
-            f"Something went wrong while extracting Payloads from a function return value. {ret}"
+            f"Something went wrong while extracting Payloads from a function return value: {ret}"
         ) from e
 
+    if _payloads:
+        logger.debug(f"{len(_payloads)} dynamic payloads received")
     for p in _payloads:
-        logger.debug(f"Function returned a Payload. Executing. {p}")
+        logger.debug(f"Executing dynamic payload: {p}")
         try:
             ret = execute_payload(p, path_enum, paths, logger, event, context, debug)
         except Exception as e:
             logger.debug(utils.exception_to_str(e))
             raise lpipe.exceptions.FailButContinue(
-                f"Failed to execute returned Payload. {p}"
+                f"Failed to execute returned Payload: {p}"
             ) from e
     return ret
 
